import ctypes
import os
from functools import wraps

import numpy as np

from mower.utils import config
from mower.utils.csleep import MowerExit
from mower.utils.device.emulator import restart_emulator
from mower.utils.device.exception import EmulatorError, GameError
from mower.utils.log import logger


class NemuIpcIncompatible(Exception):
    pass


def retry_mumuipc(func):
    @wraps(func)
    def retry_wrapper(self, *args, **kwargs):
        for _ in range(3):
            try:
                return func(self, *args, **kwargs)
            except MowerExit:
                raise
            except GameError as e:
                logger.debug_exception(e)
                config.device.check_current_focus()
            except EmulatorError as e:
                logger.debug_exception(e)
                restart_emulator()
            except Exception as e:
                logger.debug_exception(e)

    return retry_wrapper


class MuMu12IPC:
    def __init__(self):
        self.emulator_folder = config.conf.emulator.emulator_folder
        self.instanse_index = int(config.conf.emulator.index)
        self.connection = 0
        self.display_id = -1
        # 加载动态链接库
        dll_path = os.path.join(
            self.emulator_folder, "sdk", "external_renderer_ipc.dll"
        )
        try:
            self.external_renderer = ctypes.CDLL(dll_path)
        except OSError as e:
            logger.error(e)
            # OSError: [WinError 126] 找不到指定的模块。
            if not os.path.exists(dll_path):
                raise NemuIpcIncompatible(
                    f"文件不存在: {dll_path}, 请检查模拟器文件夹是否正确"
                    f"NemuIpc requires MuMu12 version >= 3.8.13, please check your version"
                )
            else:
                raise NemuIpcIncompatible(
                    f"ipc_dll={dll_path} exists, but cannot be loaded"
                )

        # 定义函数原型
        self.external_renderer.nemu_connect.argtypes = [ctypes.c_wchar_p, ctypes.c_int]
        self.external_renderer.nemu_connect.restype = ctypes.c_int

        self.external_renderer.nemu_disconnect.argtypes = [ctypes.c_int]
        self.external_renderer.nemu_disconnect.restype = None

        self.external_renderer.nemu_get_display_id.argtypes = [
            ctypes.c_int,
            ctypes.c_char_p,
            ctypes.c_int,
        ]
        self.external_renderer.nemu_get_display_id.restype = ctypes.c_int

        self.external_renderer.nemu_capture_display.argtypes = [
            ctypes.c_int,
            ctypes.c_uint,
            ctypes.c_int,
            ctypes.POINTER(ctypes.c_int),
            ctypes.POINTER(ctypes.c_int),
            ctypes.POINTER(ctypes.c_ubyte),
        ]
        self.external_renderer.nemu_capture_display.restype = ctypes.c_int
        self.external_renderer.nemu_input_event_touch_down.argtypes = [
            ctypes.c_int,
            ctypes.c_int,
            ctypes.c_int,
            ctypes.c_int,
        ]
        self.external_renderer.nemu_input_event_touch_down.restype = ctypes.c_int

        self.external_renderer.nemu_input_event_touch_up.argtypes = [
            ctypes.c_int,
            ctypes.c_int,
        ]
        self.external_renderer.nemu_input_event_touch_up.restype = ctypes.c_int

        self.external_renderer.nemu_input_event_finger_touch_down.argtypes = [
            ctypes.c_int,
            ctypes.c_int,
            ctypes.c_int,
            ctypes.c_int,
            ctypes.c_int,
        ]
        self.external_renderer.nemu_input_event_finger_touch_down.restype = ctypes.c_int

        self.external_renderer.nemu_input_event_finger_touch_up.argtypes = [
            ctypes.c_int,
            ctypes.c_int,
            ctypes.c_int,
        ]
        self.external_renderer.nemu_input_event_finger_touch_up.restype = ctypes.c_int

        self.external_renderer.nemu_input_event_key_down.argtypes = [
            ctypes.c_int,
            ctypes.c_int,
            ctypes.c_int,
        ]
        self.external_renderer.nemu_input_event_key_down.restype = ctypes.c_int

        self.external_renderer.nemu_input_event_key_up.argtypes = [
            ctypes.c_int,
            ctypes.c_int,
            ctypes.c_int,
        ]
        self.external_renderer.nemu_input_event_key_up.restype = ctypes.c_int

    def connect(self):
        "连接到 emulator"
        self.connection = self.external_renderer.nemu_connect(
            ctypes.c_wchar_p(os.path.dirname(self.emulator_folder)),
            self.instanse_index,
        )
        if self.connection == 0:
            raise EmulatorError("连接模拟器失败，请启动模拟器")
        logger.info("连接模拟器成功")

    def get_display_id(self) -> int:
        pkg_name = config.conf.APPNAME.encode("utf-8")  # 替换为实际包名
        self.display_id = self.external_renderer.nemu_get_display_id(
            self.connection, pkg_name, 0
        )
        if self.display_id < 0:
            logger.error(f"获取Display ID失败: {self.display_id}")
            raise GameError("获取Display ID失败,请先打开游戏")
        logger.info(f"获取Display ID成功: {self.display_id}")

    @retry_mumuipc
    def check_status(self):
        if self.connection == 0:
            self.connect()
        if self.display_id < 0:
            self.get_display_id()

    def capture_display(self) -> np.ndarray:
        self.check_status()
        pixels = (ctypes.c_ubyte * 8294400)()
        result = self.external_renderer.nemu_capture_display(
            self.connection,
            self.display_id,
            8294400,
            ctypes.byref(ctypes.c_int(1920)),
            ctypes.byref(ctypes.c_int(1080)),
            pixels,
        )
        if result != 0:
            self.connection = 0
            self.display_id = -1
            config.device.check_current_focus()  # 可能是游戏卡死
            return self.capture_display()

        image = np.frombuffer(pixels, dtype=np.uint8).reshape((1080, 1920, 4))[:, :, :3]
        image = np.flipud(image)  # 翻转
        return image

    def key_down(self, key_code: int):
        """按下键盘按键"""
        self.check_status()
        result = self.external_renderer.nemu_input_event_key_down(
            self.connection, self.display_id, key_code
        )
        if result != 0:
            self.connection = 0
            self.display_id = -1
            return self.key_down()

    def key_up(self, key_code: int):
        """释放键盘按键"""
        self.check_status()
        result = self.external_renderer.nemu_input_event_key_up(
            self.connection, self.display_id, key_code
        )
        if result != 0:
            self.connection = 0
            self.display_id = -1
            return self.key_up()

    # 单点触控
    def touch_down(self, x: int, y: int):
        self.check_status()
        result = self.external_renderer.nemu_input_event_touch_down(
            self.connection, self.display_id, x, y
        )
        if result != 0:
            self.connection = 0
            self.display_id = -1
            return self.touch_down()

    def touch_up(self):
        self.check_status()
        result = self.external_renderer.nemu_input_event_touch_up(
            self.connection, self.display_id
        )
        if result != 0:
            self.connection = 0
            self.display_id = -1
            return self.touch_up()

    # 多点触控
    def finger_touch_down(self, finger_id: int, x: int, y: int):
        self.check_status()
        result = self.external_renderer.nemu_input_event_finger_touch_down(
            self.connection, self.display_id, finger_id, x, y
        )
        if result != 0:
            self.connection = 0
            self.display_id = -1
            return self.finger_touch_down()

    def finger_touch_up(self, finger_id: int):
        self.check_status()
        result = self.external_renderer.nemu_input_event_finger_touch_up(
            self.connection, self.display_id, finger_id
        )
        if result != 0:
            self.connection = 0
            self.display_id = -1
            return self.finger_touch_up()
