import json

from mower.utils import config
from mower.utils.log import logger

from .utils import subprocess_run


class MuMuManager:
    manager_path = config.conf.emulator.emulator_folder + "\\MuMuManager.exe"
    index = config.conf.emulator.index

    def get_emulator_info(self):
        cmd = [self.manager_path, "info", "-v", str(self.index)]
        output = subprocess_run(cmd)

        try:
            data = json.loads(output)
            return data
        except json.JSONDecodeError as e:
            logger.error(f"JSON 解析失败: {e}")
            return {}

    def get_field_value(self, data, field_name):
        """
        从模拟器信息中提取指定字段的值
        Args:
            field_name (str): 字段名称

        Returns:
            str, int, bool, or None: 字段: 字段值或错误消息
        """
        # 获取模拟器信息
        if not data:
            logger.error("未能获取数据")
            return

        # 提取指定字段的值
        value = data.get(field_name, None)
        if value is None:
            logger.error(f"字段 '{field_name}' 不存在")
            return

        return value

    def check_android_process(self) -> bool:
        return self.get_field_value(self.get_emulator_info(), "is_android_started")

    def get_setting_info(self):
        cmd = [self.manager_path, "setting", "-v", str(self.index), "-aw"]
        output = subprocess_run(cmd)

        try:
            data = json.loads(output)
            return data
        except json.JSONDecodeError as e:
            logger.error(f"JSON 解析失败: {e}")
            return {}

    def app_kept_alive(self) -> bool:
        return self.get_field_value(self.get_setting_info(), "app_keptlive")

    def set_app_kept_alive_false(self) -> None:
        cmd = [
            self.manager_path,
            "setting",
            "-v",
            str(self.index),
            "-k",
            "app_keptlive",
            "-val",
            "false",
        ]
        subprocess_run(cmd)

    def launch(self) -> None:
        cmd = [
            self.manager_path,
            "control",
            "-v",
            str(self.index),
            "app",
            "launch",
            "-pkg",
            config.conf.APPNAME,
        ]
        subprocess_run(cmd)

    def exit(self) -> None:
        cmd = [
            self.manager_path,
            "control",
            "-v",
            str(self.index),
            "app",
            "close",
            "-pkg",
            config.conf.APPNAME,
        ]
        subprocess_run(cmd)

    def check_device_screen(self) -> bool:
        data = self.get_setting_info()
        width = int(float(self.get_field_value(data, "resolution_width.custom")))
        height = int(float(self.get_field_value(data, "resolution_height.custom")))
        dpi = int(float(self.get_field_value(data, "resolution_dpi.custom")))
        if width == 1920 and height == 1080 and dpi == 280:
            return True
        logger.error("mower-ng仅支持1920x1080分辨率、280DPI，请修改模拟器设置")
        return False
